@@ -1,9 +1,39 @@
 #' @export
+#' @rdname other-opts
+opts_Scale <- function(constructor = c("default", "next", "environment"), ...) {
+  .cstr_options("CoordCartesian", constructor = constructor[[1]], ...)
+}
+
+#' @export
+#' @method .cstr_construct Scale
 .cstr_construct.Scale <- function(x, ...) {
+  opts <- list(...)$opts$Scale %||% opts_CoordCartesian()
+  if (is_corrupted_Scale(x) || opts$constructor == "next") return(NextMethod())
+  UseMethod(".cstr_construct.Scale", structure(NA, class = opts$constructor))
+}
+
+is_corrupted_Scale <- function(x) {
+  # TODO
+  FALSE
+}
+
+#' @export
+#' @method .cstr_construct.Scale environment
+.cstr_construct.Scale.environment <- function(x, ...) {
+  .cstr_construct.environment(x, ...)
+}
+
+#' @export
+#' @method .cstr_construct.Scale default
+.cstr_construct.Scale.default <- function(x, ...) {
   # fetch caller and args from original call
-  caller <- x$call[[1]]
-  args <- as.list(x$call)[-1]
+  # here we need the ggplot subsetting method, not the low level [[
+  call <- base::`[[`(x, "call")
+  caller <- base::`[[`(x, "call")[[1]]
+  args <- as.list(call)[-1]
   fun_chr <- rlang::expr_deparse(caller)
+  # the caller might be in the form pkg::fun
+  fun_val <- eval(caller, asNamespace("ggplot2"))
 
   # fetch the actual values from the object
   values <- as.list(x)
@@ -23,13 +53,24 @@
   }
   # retrieve the defaults of the function, so we can simplify the call
   # and remove arguments that are repeating the defaults
-  fun_defaults <- defaults_arg_values(fun_chr, "ggplot2")
-  if ("trans" %in% names(args) && is.character(fun_defaults$trans)) {
-    # might be non robust, address in time
-    fun_defaults$trans <- getFromNamespace(paste0(fun_defaults$trans, "_trans"), "scales")()
+  fun_defaults <- defaults_arg_values(fun_val, "ggplot2")
+  if (length(fun_defaults)) {
+    if ("trans" %in% names(args) && is.character(fun_defaults$trans)) {
+      # might be non robust, address in time
+      fun_defaults$trans <- getFromNamespace(paste0(fun_defaults$trans, "_trans"), "scales")()
+    }
+    args_are_defaults <- mapply(identical, fun_defaults, values[names(fun_defaults)], ignore.environment = TRUE)
+    args[names(args_are_defaults)[args_are_defaults]] <- NULL
+  }
+
+  # for some reason `values` is not stored in a field of the ggproto object
+  # we can fetch it from `palette`
+  if (
+    !"values" %in% names(values) && # to be safe, might always be TRUE
+    "values" %in% names(args)
+  ) {
+    values$values <- environment(environment(x$palette)$palette)$values
   }
-  args_are_defaults <- mapply(identical, fun_defaults, values[names(fun_defaults)], ignore.environment = TRUE)
-  args[names(args_are_defaults)[args_are_defaults]] <- NULL
 
   # fetch values from ggproto object except special values
   values <- values[setdiff(names(args), c("super", "palette"))]
@@ -45,7 +86,7 @@
   }
 
   # construct values, except for `super` and `palette` that we handle specifically after
-  args[names(values)] <- lapply(values, .cstr_construct, ...)
+  args[names(values)] <- lapply(values, function(x, ...) .cstr_construct(x, ...), ...)
 
   # special case waiver as it's an empty list unfortunately matched to `.data`
   # FIXME: we should probably not match empty objects, that inclused NULL and zero length objects
@@ -62,7 +103,7 @@
       args$palette <- sprintf("scales::abs_area(%s)", environment(as.list(x)$palette)$max)
     } else if (identical(args$palette, quote(rescale_pal(range)))) {
       range_val <- environment(as.list(x)$palette)$range
-      if (identical(range_val, defaults_arg_values("rescale_pal", "scales")$range)) {
+      if (identical(range_val, defaults_arg_values(scales::rescale_pal, "scales")$range)) {
         args$palette <- "scales::rescale_pal()"
       } else {
         args$palette <- .cstr_apply(list(range_val), "scales::rescale_pal")
@@ -77,6 +118,11 @@
   }
 
   ## build call
-  fun_chr <- paste0("ggplot2::", fun_chr)
+  if (!startsWith(fun_chr, "ggplot2::")) fun_chr <- paste0("ggplot2::", fun_chr)
   .cstr_apply(args, fun = fun_chr, recurse = FALSE, ...)
 }
+
+repair_attributes_Scale <- function(x, code, ...) {
+  code
+}
+
